@@ -24,8 +24,7 @@
  *
  */
 
-#ifndef VNSI_CXSOCKET_H
-#define VNSI_CXSOCKET_H
+#pragma once
 
 #ifdef __FreeBSD__
 #include <netinet/in.h>
@@ -37,26 +36,33 @@
 #include <vdr/thread.h>
 #include <vdr/tools.h>
 
-class cxSocket
+#include "ISocket.h"
+
+class cxSocket : public ISocket
 {
-  const int m_fd;
+  int m_fd;
   cMutex m_MutexWrite;
   cPoller m_pollerRead;
   cPoller m_pollerWrite;
 
  public:
   cxSocket(int h);
-  ~cxSocket();
+  virtual ~cxSocket();
 
   cxSocket(const cxSocket &) = delete;
   cxSocket &operator=(const cxSocket &) = delete;
 
-  void Shutdown(void);
-  void LockWrite();
-  void UnlockWrite();
-  ssize_t read(void *buffer, size_t size, int timeout_ms = -1);
-  ssize_t write(const void *buffer, size_t size, int timeout_ms = -1, bool more_data = false);
   static char *ip2txt(uint32_t ip, unsigned int port, char *str);
+
+  /****************************************************************************
+   * ISocket virtual overrides
+   ***************************************************************************/
+  void Shutdown() override;
+  void lock() override;
+  void unlock() override;
+  void Invalidate() override;
+  int GetHandle() const override;
+  ssize_t read(void *buffer, size_t size, int timeout_ms = -1) override;
+  ssize_t write(const void *buffer, size_t size, int timeout_ms = -1, bool more_data = false) override;
 };
 
-#endif // VNSI_CXSOCKET_H
