@@ -31,9 +31,11 @@ int TimeshiftBufferSize = 5;
 int TimeshiftBufferFileSize = 6;
 char TimeshiftBufferDir[PATH_MAX] = "\0";
 int PlayRecording = 0;
+int GroupRecordings = 1;
 int AvoidEPGScan = 1;
 int DisableScrambleTimeout = 0;
 int DisableCamBlacklist = 0;
+int EdlMode = 0;
 
 cMenuSetupVNSI::cMenuSetupVNSI(void)
 {
@@ -55,6 +57,9 @@ cMenuSetupVNSI::cMenuSetupVNSI(void)
   newPlayRecording = PlayRecording;
   Add(new cMenuEditBoolItem( tr("Play Recording instead of live"), &newPlayRecording));
 
+  newGroupRecordings = GroupRecordings;
+  Add(new cMenuEditBoolItem( tr("Group series recordings"), &newGroupRecordings));
+
   newAvoidEPGScan = AvoidEPGScan;
   Add(new cMenuEditBoolItem( tr("Avoid EPG scan while streaming"), &newAvoidEPGScan));
 
@@ -63,6 +68,12 @@ cMenuSetupVNSI::cMenuSetupVNSI(void)
 
   newDisableCamBlacklist = DisableCamBlacklist;
   Add(new cMenuEditBoolItem( tr("Disable cam blacklist"), &newDisableCamBlacklist));
+
+  edlModesTexts[0] = tr("scene");
+  edlModesTexts[1] = tr("comskip");
+  edlModesTexts[2] = tr("cut");
+  newEdlMode = EdlMode;
+  Add(new cMenuEditStraItem( tr("EDL Mode"), &newEdlMode, 3, edlModesTexts));
 }
 
 void cMenuSetupVNSI::Store(void)
@@ -94,9 +105,13 @@ void cMenuSetupVNSI::Store(void)
 
   SetupStore(CONFNAME_PLAYRECORDING, PlayRecording = newPlayRecording);
 
+  SetupStore(CONFNAME_GROUPRECORDINGS, GroupRecordings = newGroupRecordings);
+
   SetupStore(CONFNAME_AVOIDEPGSCAN, AvoidEPGScan = newAvoidEPGScan);
 
   SetupStore(CONFNAME_DISABLESCRAMBLETIMEOUT, DisableScrambleTimeout = newDisableScrambleTimeout);
 
   SetupStore(CONFNAME_DISABLECAMBLACKLIST, DisableCamBlacklist = newDisableCamBlacklist);
+
+  SetupStore(CONFNAME_EDL, EdlMode = newEdlMode);
 }
