@@ -37,9 +37,12 @@ private:
   int newTimeshiftBufferFileSize;
   char newTimeshiftBufferDir[PATH_MAX];
   int newPlayRecording;
+  int newGroupRecordings;
   int newAvoidEPGScan;
   int newDisableScrambleTimeout;
   int newDisableCamBlacklist;
+  int newEdlMode;
+  const char *edlModesTexts[3];
 protected:
   virtual void Store(void);
 public:
