@@ -515,8 +515,6 @@ bool cVideoInput::Open(const cChannel *channel, int priority, cVideoBuffer *vide
     if (m_Device->SwitchChannel(m_Channel, false))
     {
 
-      m_Device->SetCurrentChannel(m_Channel);
-
 #if VDRVERSNUM < 20104
       m_PatFilter = new cLivePatFilter(this, m_Channel);
       m_Device->AttachFilter(m_PatFilter);
@@ -612,8 +610,8 @@ void cVideoInput::Close()
       DELETENULL(m_PatFilter);
     }
   }
-  m_Channel = NULL;
-  m_Device = NULL;
+  m_Channel = nullptr;
+  m_Device = nullptr;
   if (m_VideoBuffer)
   {
     m_VideoBuffer->AttachInput(false);
@@ -657,8 +655,12 @@ void cVideoInput::RequestRetune()
 
 cVideoInput::eReceivingStatus cVideoInput::ReceivingStatus()
 {
+  if (!IsOpen())
+    return NORMAL;
+
   if (!m_Device || !m_DummyReceiver)
     return RETUNE;
+
   if (m_RetuneRequested)
   {
     (void)m_Device->Receiving();  // wait for the receivers mutex
