@@ -449,6 +449,9 @@ void CVNSITimers::Action()
             if (event->EndTime() < time(nullptr))
               continue;
 
+            if (event->Title() == nullptr)
+              continue;
+
             std::string title(event->Title());
             std::smatch m;
             std::regex e(Convert(searchTimer.m_search));
@@ -461,7 +464,8 @@ void CVNSITimers::Action()
               {
                 if (recording->Info() != nullptr)
                 {
-                  if (strcmp(recording->Info()->Title(), event->Title()) == 0)
+                  if (recording->Info()->Title() != nullptr &&
+                      strcmp(recording->Info()->Title(), event->Title()) == 0)
                   {
                     if (recording->Info()->ShortText() != nullptr && event->ShortText() != nullptr &&
                         strcmp(recording->Info()->ShortText(), event->ShortText()) == 0)
@@ -540,7 +544,8 @@ void CVNSITimers::Action()
     if (modified)
       Timers->SetModified();
     timerState.Remove(modified);
-    SchedulesStateKey.Remove(modified);
+    if (SchedulesStateKey.InLock())
+      SchedulesStateKey.Remove(modified);
   }
 
 #endif
